#ifdef GL_ES
precision mediump float;
#endif

uniform float u_time;
uniform vec2 u_resolution;
uniform float u_now; 
uniform float u_day;

#include "lygia/math/rotate2d.glsl"


float circleSDF(vec2 c, float r, vec2 pos) {
  return sqrt(pow(pos.x - c.x, 2.) + pow(pos.y - c.y, 2.)) - r;
}

vec3 hsv2rgb(vec3 c)
{
    vec4 K = vec4(1.0, 2.0 / 3.0, 1.0 / 3.0, 3.0);
    vec3 p = abs(fract(c.xxx + K.xyz) * 6.0 - K.www);
    return c.z * mix(K.xxx, clamp(p - K.xxx, 0.0, 1.0), c.y);
}



float sdEquilateralTriangle( in vec2 p, in float r )
{
    const float k = sqrt(3.0);
    p.x = abs(p.x) - r;
    p.y = p.y + r/k;
    if( p.x+k*p.y>0.0 ) p = vec2(p.x-k*p.y,-k*p.x-p.y)/2.0;
    p.x -= clamp( p.x, -2.0*r, 0.0 );
    return -length(p)*sign(p.y);
}

float mod289(float x){return x - floor(x * (1.0 / 289.0)) * 289.0;}
vec4 mod289(vec4 x){return x - floor(x * (1.0 / 289.0)) * 289.0;}
vec4 perm(vec4 x){return mod289(((x * 34.0) + 1.0) * x);}

float noise(vec3 p){
    vec3 a = floor(p);
    vec3 d = p - a;
    d = d * d * (3.0 - 2.0 * d);

    vec4 b = a.xxyy + vec4(0.0, 1.0, 0.0, 1.0);
    vec4 k1 = perm(b.xyxy);
    vec4 k2 = perm(k1.xyxy + b.zzww);

    vec4 c = k2 + a.zzzz;
    vec4 k3 = perm(c);
    vec4 k4 = perm(c + 1.0);

    vec4 o1 = fract(k3 * (1.0 / 41.0));
    vec4 o2 = fract(k4 * (1.0 / 41.0));

    vec4 o3 = o2 * d.z + o1 * (1.0 - d.z);
    vec2 o4 = o3.yw * d.x + o3.xz * (1.0 - d.x);

    return o4.y * d.y + o4.x * (1.0 - d.y) - .5;
}

float smax1(float a, float b, float k)
{
    return log(exp(k * a) + exp(k * b)) / k;
}

// has a log2 off when they are equal,  blue
float smin2(float a, float b, float k)
{
    return -log(exp(-k * a) + exp(-k * b)) / k;
}

float smin3(float a, float b, float k)
{
    float x = exp(-k * a);
    float y = exp(-k * b);
    return (a * x + b * y) / (x + y);
}

float rand(float n){return fract(sin(n) * 43758.5453123);} 


float sdf(in vec2 p)
{
  return abs(sdEquilateralTriangle(p, 0.4));
}

vec4 color(in vec2 p) {
  float noise = noise(vec3(p * 7131.12312, 0.)) * 10.;
  return vec4(1. - noise, 1. - noise, 1. - noise, 1.);
  //return vec4(hsv2rgb(vec3((p.x + p.y + 0.1*u_time) / 10., 1., 1.)),1.);
}

void mainImage(out vec4 fragColor, in vec2 fragCoord) {
    // Normalized pixel coordinates (from -1 to 1)
    // vec2 uv = (fragCoord - u_resolution.y - .5) * 2.;
    float time = u_time + u_now * .12910298;
    vec2 uv = ((fragCoord - u_resolution * .5) / u_resolution.y) * 2.;
    vec2 uv_original = uv;
    float day_noise = rand(u_day);
    uv = uv + length(uv) * noise(vec3(uv * 4., uv.x + time / 10. + day_noise)) * 0.7 * day_noise * 2.;


    float pixelWidth = 1. / u_resolution.y;
    vec4 col = vec4(0., 0., 0., 1.);

    const float numTriangles = 4.;
    const float speedIncrement = 0.2;

    float finalSDF = 1.;

    for (float i = 0.; i < numTriangles; i++) {
      float triangleSDF = sdf(rotate2d(time * (numTriangles - i) * speedIncrement) * uv);
      finalSDF = smin3(triangleSDF, finalSDF, 8.);
      
    }
    if (((fract(finalSDF * 8.0 + time * 0.5 + u_day / 1000.) < 0.3 * (1. + sin(u_day / 100.))) || (fract(finalSDF * 10. + u_day / 1000.)< 0.3)) && (circleSDF(vec2(0.), .7, uv_original) < 0.)) {
      fragColor = color(uv);
    }
    else {
      fragColor = vec4(0., 0., 0., 1.);
  }
}

void main() {
    mainImage(gl_FragColor, gl_FragCoord.xy);
}
